precision highp float;

uniform sampler2D texShadowMap;
uniform vec3 u_v3LightColor;
#ifdef SPOT_LIGHT
uniform float u_fMaxBias;
#endif // SPOT_LIGHT

varying highp vec4 v_v4ShadowMapPosition;
varying highp vec2 v_v2TexCoordLightmap;
varying lowp vec3 v_v3Normal;
varying vec3 v_v3LightDir;

#ifdef SPOT_LIGHT
varying float v_fPerspective;
#endif

void main() 
{
#ifndef SPOT_LIGHT
	if (v_v4ShadowMapPosition.x > 1.0)
	{
		discard;
	}
	else if (v_v4ShadowMapPosition.x < 0.0)
	{
		discard;
	}
	else if (v_v4ShadowMapPosition.y > 1.0)
	{
		discard;
	}
	else if (v_v4ShadowMapPosition.y < 0.0)
	{
		discard;
	}
	else if (v_v4ShadowMapPosition.z < 0.0)
	{
		discard;
	}
	else if (v_v4ShadowMapPosition.z > 1.0)
	{
		discard;
	}
	else
#endif
	{	

#ifdef SPOT_LIGHT
	float fBias = (0.005 * tan(acos(clamp(dot(v_v3Normal, v_v3LightDir), 0.0, 1.0))));
	fBias = clamp(fBias, 0.0, u_fMaxBias);
#else
	float fMaxBias = 0.000;
	float fBias = (fMaxBias * tan(acos(clamp(dot(v_v3Normal, v_v3LightDir), 0.0, 1.0))));
	fBias = clamp(fBias, 0.0, fMaxBias);
#endif

	float fShadowZWithBias = (v_v4ShadowMapPosition.z - fBias);
	vec2 v2ShadowPos = v_v4ShadowMapPosition.xy / v_v4ShadowMapPosition.w;

	float fVisibility = 1.0;
	float amount = 0.75*0.25;
	if ( texture2D ( texShadowMap, v2ShadowPos + vec2( -0.94201624,  -0.39906216 )/1000.0 ).r  <  fShadowZWithBias ) 
		fVisibility -= amount;
	if ( texture2D( texShadowMap, v2ShadowPos + vec2(  0.94558609,  -0.76890725 )/1000.0 ).r  <  fShadowZWithBias ) 
		fVisibility -= amount;
	if ( texture2D( texShadowMap, v2ShadowPos + vec2( -0.094184101, -0.92938870 )/1000.0 ).r  <  fShadowZWithBias ) 
		fVisibility -= amount;
	if ( texture2D( texShadowMap, v2ShadowPos + vec2(  0.34495938,   0.29387760 )/1000.0 ).r  <  fShadowZWithBias ) 
		fVisibility -= amount;

	float far = 100.0;
	float farAmount = 0.25*0.25;
	if ( texture2D ( texShadowMap, v2ShadowPos + vec2( -0.94201624,  -0.39906216 )/far ).r  <  fShadowZWithBias ) 
		fVisibility -= farAmount;
	if ( texture2D( texShadowMap, v2ShadowPos + vec2(  0.94558609,  -0.76890725 )/far ).r  <  fShadowZWithBias ) 
		fVisibility -= farAmount;
	if ( texture2D( texShadowMap, v2ShadowPos + vec2( -0.094184101, -0.92938870 )/far ).r  <  fShadowZWithBias ) 
		fVisibility -= farAmount;
	if ( texture2D( texShadowMap, v2ShadowPos + vec2(  0.34495938,   0.29387760 )/far ).r  <  fShadowZWithBias ) 
		fVisibility -= farAmount;

	float fAngleMod = dot(v_v3Normal, v_v3LightDir);
	fVisibility *= clamp(-exp(fAngleMod) + 2.0, 0.0, 1.0);

#ifdef SPOT_LIGHT
	vec2 v2ShadowDist = (v_v4ShadowMapPosition.xy * 2.0) - 1.0;
	float fI = ((2.0 - dot(v2ShadowDist, v2ShadowDist)) * 0.5) - 0.5;
	fI = clamp(fI, 0.0, 1.0);
	gl_FragColor = vec4(u_v3LightColor, fVisibility * fI);	
#else
	gl_FragColor = vec4(u_v3LightColor, fVisibility);	
#endif
	}
}
