
precision lowp float;

uniform sampler2D tex;
varying vec2 v_v2TexCoord;
varying vec4 v_v4Color;

	
#ifdef REWIND
uniform lowp float u_fRewindEffect;
#endif

#ifndef DETAIL_LOW
	uniform lowp samplerCube texCubeMap;
	uniform lowp float u_fSpecular;
#endif

#ifdef DETAIL_MEDIUM
	varying mediump vec3 v_v3Reflection;
#endif

#ifdef DETAIL_HIGH
	varying lowp vec3 v_v3Normal;
	varying mediump vec3 v_v3WorldPosition;
	uniform lowp float u_fBrightness;
	uniform mediump vec3 u_v3EyePosition;
#endif

void main() 
{
	vec4 v4Texture = texture2D(tex, v_v2TexCoord) * v_v4Color;
		
	#ifdef DETAIL_HIGH
		mediump vec3 v3Incident = v_v3WorldPosition - u_v3EyePosition;
		mediump vec3 v3Reflection = v3Incident - 2.0 * dot(v3Incident, v_v3Normal) * v_v3Normal / dot(v_v3Normal, v_v3Normal);	
		lowp vec4 v4TextureCube = textureCube(texCubeMap, v3Reflection);

		lowp float fSpec = u_fSpecular * ((v4Texture.r + v4Texture.g + v4Texture.b) * 0.333);

		lowp vec3 v3Out = v4Texture.rgb + v4TextureCube.rgb * u_fSpecular * fSpec;
	#endif

	#ifdef DETAIL_MEDIUM
		lowp vec4 v4TextureCube = textureCube(texCubeMap, v_v3Reflection);
		lowp float fSpec = u_fSpecular * ((v4Texture.r + v4Texture.g + v4Texture.b) * 0.333);

		lowp vec3 v3Out = v4Texture.rgb + v4TextureCube.rgb * fSpec;
	#endif

	#ifdef DETAIL_LOW
		lowp vec3 v3Out = v4Texture.rgb;	
	#endif
	
	#ifdef REWIND
		v3Out = v3Out * (1.0 - u_fRewindEffect) + v4Texture.b * u_fRewindEffect;
	#endif
	
	#ifdef DETAIL_HIGH
		lowp vec3 v3Tmp = 1.0 - v3Out;
		gl_FragColor.rgb = v3Out * (2.0 - u_fBrightness) + (1.0 - v3Tmp * v3Tmp * v3Tmp) * (u_fBrightness - 1.0);	
	#else
		gl_FragColor.rgb = v3Out;
	#endif

	gl_FragColor.a = v4Texture.a;
}
