

uniform mat4 u_m44ModelViewProjection;
uniform float u_fZOffset;
uniform float u_fZClamp;
uniform float u_fMinZOffset; 
uniform float u_fAlphaLevel;

attribute vec4 a_v4Position;
attribute vec2 a_v2TexCoord;
attribute vec4 a_v4Color;

varying vec2 v_v2TexCoord;
varying vec4 v_v4Color;

#ifndef DETAIL_LOW
	attribute vec3 a_v3Normal;
	uniform mat3 u_m33SpecularRotation;
#endif

#ifdef DETAIL_HIGH
	varying lowp vec3 v_v3Normal;
	varying mediump vec3 v_v3WorldPosition;
#endif

#ifdef DETAIL_MEDIUM
	varying mediump vec3 v_v3Reflection;
	uniform mediump vec3 u_v3EyePosition;
#endif

void main() 
{
	v_v2TexCoord = a_v2TexCoord;
	v_v4Color = a_v4Color * (1.0 / 255.0);
	v_v4Color.a *= u_fAlphaLevel;	
	
	gl_Position = u_m44ModelViewProjection * a_v4Position;
	float fDiv = gl_Position.w;
	if (fDiv < u_fZClamp)
		fDiv = u_fZClamp;
	float zOffset = u_fZOffset / fDiv;
	if (zOffset < u_fMinZOffset)
		zOffset = u_fMinZOffset;
	gl_Position.z -= zOffset;

	#ifdef DETAIL_HIGH
		v_v3Normal = a_v3Normal * u_m33SpecularRotation;
		v_v3WorldPosition = a_v4Position.xyz * u_m33SpecularRotation;
	#endif

	#ifdef DETAIL_MEDIUM
		vec3 v3Normal = a_v3Normal * u_m33SpecularRotation;			
		mediump vec3 v3Incident = a_v4Position.xyz * u_m33SpecularRotation - u_v3EyePosition;
		mediump vec3 v3Reflection = v3Incident - 2.0 * dot(v3Incident, v3Normal) * v3Normal / dot(v3Normal, v3Normal);		
		v_v3Reflection = v3Reflection;	
	#endif
}


