
uniform mat4 u_m44ModelViewProjection;


attribute vec4 a_v4Position;
attribute vec2 a_v2TexCoord;
attribute vec2 a_v2TexCoord3;

#ifdef TRANSPARENT
attribute float a_fDistanceScaleFade;
#endif

varying highp vec2 v_v2TexCoord;
varying mediump vec2 v_v2LightTexCoord;

#ifdef TRANSPARENT
varying mediump float v_fDistanceFade;
#endif


#ifndef DETAIL_LOW
	uniform mat3 u_m33SpecularRotation;

	attribute vec3 a_v3Normal;

	#ifdef COMPOSIT_CHANNELS
		attribute vec4 a_v4Color;
		attribute vec4 a_v4Color2;

		varying mediump vec4 v_v4Color;
		varying mediump vec4 v_v4Color2;
	#endif

	#ifdef DETAIL_HIGH
		varying lowp vec3 v_v3Normal;
		varying mediump vec3 v_v3WorldPosition;
	#endif

	#ifdef DETAIL_MEDIUM
		varying mediump vec3 v_v3Reflection;
		uniform mediump vec3 u_v3EyePosition;
	#endif
#endif //#ifndef DETAIL_LOW

#if defined (DETAIL_LOW) && defined (TRANSPARENT)
	uniform mediump vec3 u_v3EyePosition;
#endif




#ifdef Z_OFFSET
	uniform float u_fZOffset;
	uniform float u_fZClamp;
	uniform float u_fMinZOffset; 
#endif

void main() 
{
	v_v2TexCoord = a_v2TexCoord;
	v_v2LightTexCoord = a_v2TexCoord3;
	

	gl_Position = u_m44ModelViewProjection * a_v4Position;

	#ifdef TRANSPARENT
		#ifdef DETAIL_HIGH
			v_fDistanceFade = 1.0;
		#else
			//v_fDistanceFade = (5.0 - length(gl_Position.xyz) * 0.08 + a_fDistanceScaleFade * (5.0 * 0.08) );
			v_fDistanceFade = (5.0 - length(u_v3EyePosition - a_v4Position.xyz) * 0.08 + a_fDistanceScaleFade * (5.0 * 0.08) );
		#endif
	#endif
	
	#ifndef DETAIL_LOW

		#ifdef COMPOSIT_CHANNELS
			v_v4Color = a_v4Color * (1.0 / 255.0);
			v_v4Color2 = a_v4Color2 * (1.0 / 255.0);
		#endif
		
		#ifdef DETAIL_MEDIUM
			vec3 v3Normal = a_v3Normal * u_m33SpecularRotation;			
			mediump vec3 v3Incident = a_v4Position.xyz * u_m33SpecularRotation - u_v3EyePosition;
			mediump vec3 v3Reflection = v3Incident - 2.0 * dot(v3Incident, v3Normal) * v3Normal / dot(v3Normal, v3Normal);		
			v_v3Reflection = v3Reflection;	
		#endif

		#ifdef DETAIL_HIGH
			v_v3Normal = a_v3Normal * u_m33SpecularRotation;
			v_v3WorldPosition = a_v4Position.xyz * u_m33SpecularRotation;
		#endif
	#endif //#ifndef DETAIL_LOW

	#ifdef Z_OFFSET
		float fDiv = gl_Position.w;
		if (fDiv < u_fZClamp)
			fDiv = u_fZClamp;
		float zOffset = u_fZOffset / fDiv;
		if (zOffset < u_fMinZOffset)
			zOffset = u_fMinZOffset;

		#ifndef DETAIL_LOW
			#ifdef Z_OFFSET_IGNORE_DOWNWARD_FACES
				float fNormalModifier = clamp((a_v3Normal.y + 0.1) * 10000.0, 0.0, 1.0);
				gl_Position.z -= (zOffset * fNormalModifier);
			#else
				gl_Position.z -= zOffset;
			#endif
		#else
			gl_Position.z -= zOffset;
		#endif
	#endif	
	
}

