uniform mat4 u_m44ModelViewProjection;
uniform vec3 u_v3EyePos;
uniform vec3 u_v3LightDir;

varying vec4 v_v4Colour;
uniform vec4 u_v4Color;

#ifdef TRANSLUCENT
uniform vec3 u_v3TranslucentWobbleOffset;
uniform vec3 u_v3TranslucencyNormal;
uniform float u_fTranslucencyDistance;
#endif

attribute vec4 a_v4Position;
attribute vec2 a_v2TexCoord;
attribute vec2 a_v2TexCoord2;

#ifdef TANGENT_NORMAL_MAP

attribute highp vec3 a_v3MatrixX;
attribute highp vec3 a_v3MatrixY;
attribute highp vec3 a_v3MatrixZ;

highp vec3 v3MatrixX;
highp vec3 v3MatrixY;
highp vec3 v3MatrixZ;

//varying highp vec3 v_v3Normal;

#else
attribute vec3 a_v3Normal;
varying vec3 v_v3Normal;

#endif
varying highp vec3 v_v3Light_Transformed;

varying vec2 v_v2TexCoord;
varying vec2 v_v2TexCoord2;

varying vec3 v_v3WorldPos;
varying vec3 v_v3ViewDir;

varying vec3 v_v3LightDir;
varying vec3 v_v3LightColor;


void main() 
{
	vec4 v4Position = a_v4Position;
	v4Position.w = 1.0;
	v_v3WorldPos = a_v4Position.xyz;

	v_v2TexCoord = a_v2TexCoord;

	v_v3LightDir = u_v3LightDir;
	
	v_v3LightColor = 1.0*vec3( 255.0, 240.0, 230.0) * (1.0 / 255.0);
	
    vec3 v3ViewDir = normalize(vec3(u_m44ModelViewProjection * v4Position) - u_v3EyePos);

	v_v2TexCoord2=a_v2TexCoord2; 

#ifdef TANGENT_NORMAL_MAP

	v3MatrixX = mat3(u_m44ModelViewProjection)*a_v3MatrixX;
    v3MatrixY = mat3(u_m44ModelViewProjection)*a_v3MatrixY;
  	v3MatrixZ = mat3(u_m44ModelViewProjection)*a_v3MatrixZ;

	vec3 v3LightDir = v_v3LightDir;
	v3LightDir =  normalize(v_v3LightDir);

	v_v3Light_Transformed.x = dot(v3LightDir, v3MatrixX);
	v_v3Light_Transformed.y = dot(v3LightDir, v3MatrixY);
	v_v3Light_Transformed.z = dot(v3LightDir, v3MatrixZ);
	
    
	v_v3ViewDir.x = dot(v3ViewDir, v3MatrixX);
	v_v3ViewDir.y = dot(v3ViewDir, v3MatrixY);
	v_v3ViewDir.z = dot(v3ViewDir, v3MatrixZ);
    
    v_v3Light_Transformed = normalize(v_v3Light_Transformed);
    v_v3ViewDir = normalize(v_v3ViewDir);
    
#else
	v_v3Light_Transformed =  normalize(v_v3LightDir);

	v_v3Normal = normalize( mat3(u_m44ModelViewProjection) * a_v3Normal);
    v_v3ViewDir = v3ViewDir;
#endif

	v_v4Colour = u_v4Color;
#ifdef TRANSLUCENT

	vec4 v3Position = v4Position;

	vec3 v3Position2 = a_v4Position.xyz * 0.1 + u_v3TranslucentWobbleOffset * -2.0;

	v3Position.x += sin(v3Position2.y * 2.3) * sin(v3Position2.z * 1.1) * 4.1 + sin(v3Position2.y * 5.3) * sin(v3Position2.z * 3.41) * 4.5;
	v3Position.y += sin(v3Position2.z * 3.1) * sin(v3Position2.x * 2.7) * 4.3 + sin(v3Position2.z * 2.34) * sin(v3Position2.x * 2.64) * 4.6;
	v3Position.z += sin(v3Position2.x * 4.3) * sin(v3Position2.y * 2.3) * 4.4 + sin(v3Position2.x * 1.33) * sin(v3Position2.y * 4.33) * 4.7;


	float fLerp = 0.8 + 0.07 * (sin(v3Position2.x * 1.3) + sin(v3Position2.z * 0.95) + sin(v3Position2.x * 5.1) + sin(v3Position2.x * 3.1) + sin(v3Position2.y * 1.1) + sin(v3Position2.y * 0.7) + sin(v3Position2.z * 3.7));

	
	v_v4Colour.w += (1.0 - v_v4Colour.w) * fLerp;
    
    
    float fDepthTranslucency = dot(u_v3TranslucencyNormal, a_v4Position.xyz * 0.01) - u_fTranslucencyDistance;
   // fDepthTranslucency *= 2.0;
    if (fDepthTranslucency > 0.0)
    {
        if (fDepthTranslucency > 1.0)
            fDepthTranslucency = 1.0;
        
        v_v4Colour.rgb *= 1.0 - fDepthTranslucency;
        v_v4Colour.w += fDepthTranslucency * (1.0 - v_v4Colour.w);
    
    }

    
    
    
   // v_v4Colour.rgb = vec3(1.0);
	gl_Position = u_m44ModelViewProjection * v3Position;

#else
    
	gl_Position = u_m44ModelViewProjection * v4Position;

#endif

	
}
